@@ -16,15 +16,30 @@ ldns_dnssec_rrs_new()
 	return new_rrs;
 }
 
+INLINE void
+ldns_dnssec_rrs_free_internal(ldns_dnssec_rrs *rrs, int deep)
+{
+	ldns_dnssec_rrs *next;
+	while (rrs) {
+		next = rrs->next;
+		if (deep) {
+			ldns_rr_free(rrs->rr);
+		}
+		LDNS_FREE(rrs);
+		rrs = next;
+	}
+}
+
 void
 ldns_dnssec_rrs_free(ldns_dnssec_rrs *rrs)
 {
-	if (rrs) {
-		if (rrs->next) {
-			ldns_dnssec_rrs_free(rrs->next);
-		}
-		LDNS_FREE(rrs);
-	}
+	ldns_dnssec_rrs_free_internal(rrs, 0);
+}
+
+void
+ldns_dnssec_rrs_deep_free(ldns_dnssec_rrs *rrs)
+{
+	ldns_dnssec_rrs_free_internal(rrs, 1);
 }
 
 ldns_status
@@ -88,23 +103,35 @@ ldns_dnssec_rrsets_new()
 	return new_rrsets;
 }
 
-void
-ldns_dnssec_rrsets_free(ldns_dnssec_rrsets *rrsets)
+INLINE void
+ldns_dnssec_rrsets_free_internal(ldns_dnssec_rrsets *rrsets, int deep)
 {
 	if (rrsets) {
 		if (rrsets->rrs) {
-			ldns_dnssec_rrs_free(rrsets->rrs);
+			ldns_dnssec_rrs_free_internal(rrsets->rrs, deep);
 		}
 		if (rrsets->next) {
-			ldns_dnssec_rrsets_free(rrsets->next);
+			ldns_dnssec_rrsets_free_internal(rrsets->next, deep);
 		}
 		if (rrsets->signatures) {
-			ldns_dnssec_rrs_free(rrsets->signatures);
+			ldns_dnssec_rrs_free_internal(rrsets->signatures, deep);
 		}
 		LDNS_FREE(rrsets);
 	}
 }
 
+void
+ldns_dnssec_rrsets_free(ldns_dnssec_rrsets *rrsets)
+{
+	ldns_dnssec_rrsets_free_internal(rrsets, 0);
+}
+
+void
+ldns_dnssec_rrsets_deep_free(ldns_dnssec_rrsets *rrsets)
+{
+	ldns_dnssec_rrsets_free_internal(rrsets, 1);
+}
+
 ldns_rr_type
 ldns_dnssec_rrsets_type(ldns_dnssec_rrsets *rrsets)
 {
@@ -296,23 +323,39 @@ ldns_dnssec_name_new_frm_rr(ldns_rr *rr)
 	return new_name;
 }
 
-void
-ldns_dnssec_name_free(ldns_dnssec_name *name)
+INLINE void
+ldns_dnssec_name_free_internal(ldns_dnssec_name *name,
+                               int deep)
 {
 	if (name) {
 		if (name->name_alloced) {
 			ldns_rdf_deep_free(name->name);
 		}
 		if (name->rrsets) {
-			ldns_dnssec_rrsets_free(name->rrsets);
+			ldns_dnssec_rrsets_free_internal(name->rrsets, deep);
+		}
+		if (name->nsec && deep) {
+			ldns_rr_free(name->nsec);
 		}
 		if (name->nsec_signatures) {
-			ldns_dnssec_rrs_free(name->nsec_signatures);
+			ldns_dnssec_rrs_free_internal(name->nsec_signatures, deep);
 		}
 		LDNS_FREE(name);
 	}
 }
 
+void
+ldns_dnssec_name_free(ldns_dnssec_name *name)
+{
+  ldns_dnssec_name_free_internal(name, 0);
+}
+
+void
+ldns_dnssec_name_deep_free(ldns_dnssec_name *name)
+{
+  ldns_dnssec_name_free_internal(name, 1);
+}
+
 ldns_rdf *
 ldns_dnssec_name_name(ldns_dnssec_name *name)
 {
@@ -512,11 +555,18 @@ ldns_dnssec_zone_new()
 
 void
 ldns_dnssec_name_node_free(ldns_rbnode_t *node, void *arg) {
-	arg = arg;
+	(void) arg;
 	ldns_dnssec_name_free((ldns_dnssec_name *)node->data);
 	free(node);
 }
 
+void
+ldns_dnssec_name_node_deep_free(ldns_rbnode_t *node, void *arg) {
+	(void) arg;
+	ldns_dnssec_name_deep_free((ldns_dnssec_name *)node->data);
+	free(node);
+}
+
 void
 ldns_dnssec_zone_free(ldns_dnssec_zone *zone)
 {
@@ -532,6 +582,21 @@ ldns_dnssec_zone_free(ldns_dnssec_zone *zone)
 	}
 }
 
+void
+ldns_dnssec_zone_deep_free(ldns_dnssec_zone *zone)
+{
+	if (zone) {
+		if (zone->names) {
+			/* destroy all name structures within the tree */
+			ldns_traverse_postorder(zone->names,
+						    ldns_dnssec_name_node_deep_free,
+						    NULL);
+			free(zone->names);
+		}
+		LDNS_FREE(zone);
+	}
+}
+
 /* use for dname comparison in tree */
 int
 ldns_dname_compare_v(const void *a, const void *b) {
@@ -580,11 +645,11 @@ ldns_dnssec_zone_add_rr(ldns_dnssec_zone *zone, ldns_rr *rr)
 	if (!zone->names) {
 		zone->names = ldns_rbtree_create(ldns_dname_compare_v);
 	}
-	
+
 	/* we need the original of the hashed name if this is
 	   an NSEC3, or an RRSIG that covers an NSEC3 */
 	if (ldns_rr_get_type(rr) == LDNS_RR_TYPE_RRSIG) {
-		type_covered = ldns_rdf2rr_type(ldns_rr_rrsig_typecovered(rr));;
+		type_covered = ldns_rdf2rr_type(ldns_rr_rrsig_typecovered(rr));
 	}
 	if (ldns_rr_get_type(rr) == LDNS_RR_TYPE_NSEC3 ||
 	    type_covered == LDNS_RR_TYPE_NSEC3) {
