@@ -2,7 +2,7 @@
  * key2ds transforms a public key into its DS
  * It (currently) prints out the public key
  *
- * (c) NLnet Labs, 2005
+ * (c) NLnet Labs, 2005 - 2008
  * See the file LICENSE for the license
  */
 
@@ -51,7 +51,7 @@ main(int argc, char *argv[])
 		} 
 		if (strcmp(argv[0], "-2") == 0) {
 		        #ifndef SHA256_DIGEST_LENGTH
-		          fprintf(stderr, "Error: Crypto library does not support SHA256 digests!\n");
+		          fprintf(stderr, "Error: Crypto library does not support SHA256 digests!");
 		        #endif
 			h = LDNS_SHA256;
 		}
@@ -75,7 +75,7 @@ main(int argc, char *argv[])
 	}
 
 	result = ldns_rr_new_frm_fp(&k, keyfp, 0, &origin, NULL);
-	while (result == LDNS_STATUS_SYNTAX_ORIGIN || result == LDNS_STATUS_SYNTAX_TTL) {
+	while (result == LDNS_STATUS_SYNTAX_ORIGIN) {
 		result = ldns_rr_new_frm_fp(&k, keyfp, 0, &origin, NULL);
 	}
 	if (result != LDNS_STATUS_OK) {
