@@ -2,7 +2,7 @@
  * keygen is a small programs that generate a dnskey and private key
  * for a particular domain.
  *
- * (c) NLnet Labs, 2005
+ * (c) NLnet Labs, 2005 - 2008
  * See the file LICENSE for the license
  */
 
@@ -14,12 +14,12 @@
 
 void
 usage(FILE *fp, char *prog) {
-	fprintf(fp, "%s [-D|-R] [-b bits] [-r /dev/random] [-v] domain\n", prog);
+	fprintf(fp, "%s -a <algorithm> [-b bits] [-r /dev/random] [-v] domain\n",
+		   prog);
 	fprintf(fp, "  generate a new key pair for domain\n");
-	fprintf(fp, "  -D\tgenerate a DSA key\n");
-	fprintf(fp, "  -R\tgenerate an RSA key\n");
-	fprintf(fp, "  -H\tgenerate an HMAC-MD5 key (for TSIG)\n");
-	fprintf(fp, "  -k\tset the flags to 257; key signing key\n");
+	fprintf(fp, "  -a <alg>\tuse the specified algorithm (-a list to");
+	fprintf(fp, " show a list)\n");
+	fprintf(fp, "  -k\t\tset the flags to 257; key signing key\n");
 	fprintf(fp, "  -b <bits>\tspecify the keylength\n");
 	fprintf(fp, "  -r <random>\tspecify a random device (defaults to /dev/random)\n");
 	fprintf(fp, "\t\tto seed the random generator with\n");
@@ -31,6 +31,18 @@ usage(FILE *fp, char *prog) {
 	fprintf(fp, "  The base name (K<name>+<alg>+<id> will be printed to stdout\n");
 }
 
+void
+show_algorithms(FILE *out)
+{
+	ldns_lookup_table *lt = ldns_signing_algorithms;
+	fprintf(out, "Possible algorithms:\n");
+
+	while (lt->name) {
+		fprintf(out, "%s\n", lt->name);
+		lt++;
+	}
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -53,32 +65,36 @@ main(int argc, char *argv[])
 	ldns_key *key;
 	ldns_rr *ds;
 
+	ldns_lookup_table *lt;
+
 	prog = strdup(argv[0]);
 	algorithm = 0;
 	random = NULL;
 	ksk = false; /* don't create a ksk per default */
-	
-	while ((c = getopt(argc, argv, "DRHkb:r:v")) != -1) {
+
+	while ((c = getopt(argc, argv, "a:kb:r:v25")) != -1) {
 		switch (c) {
-		case 'D':
+		case 'a':
 			if (algorithm != 0) {
-				fprintf(stderr, "%s: %s", prog, "Only one of -D, -A or -H is allowed\n");
+				fprintf(stderr, "The -a argument can only be used once\n");
+				exit(1);
+			}
+			if (strncmp(optarg, "list", 5) == 0) {
+				show_algorithms(stdout);
+				exit(EXIT_SUCCESS);
+			}
+			lt = ldns_signing_algorithms;
+			while (lt->name) {
+				if (strncasecmp(lt->name, optarg, strlen(lt->name)) == 0) {
+					algorithm = lt->id;
+				}
+				lt++;
+			}
+			if (algorithm == 0) {
+				fprintf(stderr, "Algorithm %s not found\n", optarg);
+				show_algorithms(stderr);
 				exit(EXIT_FAILURE);
 			}
-			algorithm = LDNS_SIGN_DSA;
-			break;
-		case 'R':
-			if (algorithm != 0) {
-				fprintf(stderr, "%s: %s", prog, "Only one of -D, -A or -H is allowed\n");
-				exit(EXIT_FAILURE);
-			}
-			algorithm = LDNS_SIGN_RSASHA1;
-			break;
-                case 'H':
-			if (algorithm != 0) {
-				fprintf(stderr, "%s: %s", prog, "Only one of -D, -A or -H is allowed\n");
-			}
-			algorithm = LDNS_SIGN_HMACMD5;
 			break;
 		case 'b':
 			bits = (uint16_t) atoi(optarg);
@@ -110,13 +126,14 @@ main(int argc, char *argv[])
 	argv += optind;
 
 	if (algorithm == 0) {
-		algorithm = LDNS_SIGN_RSASHA1; /* default to RSA SHA1 */
+		printf("Please use the -a argument to provide an algorithm\n");
+		exit(1);
 	}
 
 	if (argc != 1) {
 		usage(stderr, prog);
 		exit(EXIT_FAILURE);
-	} 
+	}
 	free(prog);
 
 	/* check whether key size is within RFC boundaries */
@@ -137,9 +154,12 @@ main(int argc, char *argv[])
 		}
 		break;
 	case LDNS_SIGN_HMACMD5:
+	case LDNS_SIGN_HMACSHA1:
+	case LDNS_SIGN_HMACSHA256:
+	default:
 		break;
 	}
-	
+
 	if (!random) {
 		random = fopen("/dev/random", "r");
 		if (!random) {
@@ -156,7 +176,7 @@ main(int argc, char *argv[])
 
 	/* generate a new key */
 	key = ldns_key_new_frm_algorithm(algorithm, bits);
-	
+
 	/* set the owner name in the key - this is a /seperate/ step */
 	ldns_key_set_pubkey_owner(key, domain);
 
@@ -217,7 +237,9 @@ main(int argc, char *argv[])
 	}
 	
 	/* print the DS to .ds */
-	if (algorithm != LDNS_SIGN_HMACMD5) {
+	if (algorithm != LDNS_SIGN_HMACMD5 &&
+		algorithm != LDNS_SIGN_HMACSHA1 &&
+		algorithm != LDNS_SIGN_HMACSHA256) {
 		filename = LDNS_XMALLOC(char, strlen(owner) + 16);
 		snprintf(filename, strlen(owner) + 15, "K%s+%03u+%05u.ds", owner, algorithm, (unsigned int) ldns_key_keytag(key));
 		file = fopen(filename, "w");
@@ -235,21 +257,6 @@ main(int argc, char *argv[])
 			LDNS_FREE(filename);
 		}
 	}	
-#if 0
-	/* TEMP: create PEM format too */
-	filename = LDNS_XMALLOC(char, strlen(owner) + 17);
-	snprintf(filename, strlen(owner) + 16, "K%s+%03u+%05u.pem", owner, algorithm, ldns_key_keytag(key));
-	file = fopen(filename, "w");
-	if (!file) {
-		fprintf(stderr, "Unable to open %s: %s\n", filename, strerror(errno));
-		exit(EXIT_FAILURE);
-	} else {
-				
-PEM_write_DSAPrivateKey(file, key->_key.dsa, NULL, NULL, 0, NULL, NULL);
-		fclose(file);
-		LDNS_FREE(filename);
-	}
-#endif
 
 	fprintf(stdout, "K%s+%03u+%05u\n", owner, algorithm, (unsigned int) ldns_key_keytag(key));
 	ldns_key_deep_free(key);
