@@ -1107,7 +1107,7 @@ ldns_rr2buffer_str(ldns_buffer *output, const ldns_rr *rr)
 								ldns_rr_dnskey_key_size(rr)); 
 						break;
 					} 
-					if (flags == 256 || flags == 385) {
+					if (flags == 257 || flags == 385) {
 						ldns_buffer_printf(output, 
 								" ;{id = %d (ksk), size = %db}", 
 								ldns_calc_keytag(rr),
@@ -1132,6 +1132,11 @@ ldns_rr2buffer_str(ldns_buffer *output, const ldns_rr *rr)
 						LDNS_FREE(babble);
 					}
 					break;
+				case LDNS_RR_TYPE_NSEC3:
+					if (ldns_nsec3_optout(rr)) {
+						ldns_buffer_printf(output, " ; flags: optout");
+					}
+					break;
 				default:
 					break;
 
@@ -1282,9 +1287,11 @@ ldns_pkt2buffer_str(ldns_buffer *output, const ldns_pkt *pkt)
 			if (ldns_pkt_edns_do(pkt)) {
 				ldns_buffer_printf(output, " do");
 			}
+			/* the extended rcode is the value set, shifted four bits,
+			 * and or'd with the original rcode */
 			if (ldns_pkt_edns_extended_rcode(pkt)) {
 				ldns_buffer_printf(output, " ; ext-rcode: %d",
-					ldns_pkt_edns_extended_rcode(pkt));
+					(ldns_pkt_edns_extended_rcode(pkt) << 4 | ldns_pkt_get_rcode(pkt)));
 			}
 			ldns_buffer_printf(output, " ; udp: %u\n",
 					   ldns_pkt_edns_udp_size(pkt));
