@@ -69,7 +69,9 @@ ldns_key_new()
 		ldns_key_set_inception(newkey, 0);
 		ldns_key_set_expiration(newkey, 0);
 		ldns_key_set_pubkey_owner(newkey, NULL);
+		ldns_key_set_evp_key(newkey, NULL);
 		ldns_key_set_hmac_key(newkey, NULL);
+		ldns_key_set_external_key(newkey, NULL);
 		return newkey;
 	}
 	return NULL;
@@ -392,6 +394,7 @@ ldns_key_new_frm_fp_rsa_l(FILE *f, int *line_nr)
 	return rsa;
 
 error:
+	RSA_free(rsa);
 	LDNS_FREE(d);
 	LDNS_FREE(buf);
 	return NULL;
@@ -652,6 +655,12 @@ ldns_key_set_hmac_size(ldns_key *k, size_t hmac_size)
 	k->_key.hmac.size = hmac_size;
 }
 
+void
+ldns_key_set_external_key(ldns_key *k, void *external_key)
+{
+	k->_key.external_key = external_key;
+}
+
 void
 ldns_key_set_origttl(ldns_key *k, uint32_t t)
 {
@@ -731,25 +740,47 @@ ldns_key_evp_key(const ldns_key *k)
 RSA *
 ldns_key_rsa_key(const ldns_key *k)
 {
-	return EVP_PKEY_get1_RSA(k->_key.key);
+	if (k->_key.key) {
+		return EVP_PKEY_get1_RSA(k->_key.key);
+	} else {
+		return NULL;
+	}
 }
 
 DSA *
 ldns_key_dsa_key(const ldns_key *k)
 {
-	return EVP_PKEY_get1_DSA(k->_key.key);
+	if (k->_key.key) {
+		return EVP_PKEY_get1_DSA(k->_key.key);
+	} else {
+		return NULL;
+	}
 }
 
 unsigned char *
 ldns_key_hmac_key(const ldns_key *k)
 {
-	return k->_key.hmac.key;
+	if (k->_key.hmac.key) {
+		return k->_key.hmac.key;
+	} else {
+		return NULL;
+	}
 }
 
 size_t
 ldns_key_hmac_size(const ldns_key *k)
 {
-	return k->_key.hmac.size;
+	if (k->_key.hmac.size) {
+		return k->_key.hmac.size;
+	} else {
+		return 0;
+	}
+}
+
+void *
+ldns_key_external_key(const ldns_key *k)
+{
+	return k->_key.external_key;
 }
 
 uint32_t
@@ -926,21 +957,17 @@ ldns_key2rr(const ldns_key *k)
 	 */
 	ldns_rr *pubkey;
 	ldns_rdf *keybin;
-	unsigned char *bin;
+	unsigned char *bin = NULL;
 	uint16_t size = 0;
 	RSA *rsa = NULL;
 	DSA *dsa = NULL;
+	int internal_data = 0;
 
 	pubkey = ldns_rr_new();
 	if (!k) {
 		return NULL;
 	}
 
-	bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
-	if (!bin) {
-		return NULL;
-	}
-
 	switch (ldns_key_algorithm(k)) {
 	case LDNS_SIGN_HMACMD5:
 	case LDNS_SIGN_HMACSHA1:
@@ -975,13 +1002,16 @@ ldns_key2rr(const ldns_key *k)
 			ldns_rr_push_rdf(pubkey,
 						  ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, ldns_key_algorithm(k)));
 			rsa =  ldns_key_rsa_key(k);
-			if (!rsa) {
-				return NULL;
-			} else {
+			if (rsa) {
+				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
+				if (!bin) {
+					return NULL;
+				}
 				if (!ldns_key_rsa2bin(bin, rsa, &size)) {
 					return NULL;
 				}
 				RSA_free(rsa);
+				internal_data = 1;
 			}
 			size++;
 			break;
@@ -989,38 +1019,55 @@ ldns_key2rr(const ldns_key *k)
 			ldns_rr_push_rdf(pubkey,
 					ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, LDNS_DSA));
 			dsa = ldns_key_dsa_key(k);
-			if (!dsa) {
-				return NULL;
-			} else {
+			if (dsa) {
+				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
+				if (!bin) {
+					return NULL;
+				}
 				if (!ldns_key_dsa2bin(bin, dsa, &size)) {
 					return NULL;
 				}
 				DSA_free(dsa);
+				internal_data = 1;
 			}
 			break;
 		case LDNS_DSA_NSEC3:
 			ldns_rr_push_rdf(pubkey,
 					ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG, LDNS_DSA_NSEC3));
-			if (!ldns_key_dsa2bin(bin, ldns_key_dsa_key(k), &size)) {
-				return NULL;
+			dsa = ldns_key_dsa_key(k);
+			if (dsa) {
+				bin = LDNS_XMALLOC(unsigned char, LDNS_MAX_KEYLEN);
+				if (!bin) {
+					return NULL;
+				}
+				if (!ldns_key_dsa2bin(bin, dsa, &size)) {
+					return NULL;
+				}
+				DSA_free(dsa);
+				internal_data = 1;
 			}
 			break;
 		case LDNS_SIGN_HMACMD5:
 		case LDNS_SIGN_HMACSHA1:
 		case LDNS_SIGN_HMACSHA256:
-			/* tja */
+			bin = LDNS_XMALLOC(unsigned char, size);
+			if (!bin) {
+				return NULL;
+			}
 			ldns_rr_push_rdf(pubkey,
-                                        ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG,
-						ldns_key_algorithm(k)));
-                        size = ldns_key_hmac_size(k);
-			bin = LDNS_XREALLOC(bin, unsigned char, size);
+			                 ldns_native2rdf_int8(LDNS_RDF_TYPE_ALG,
+			                 ldns_key_algorithm(k)));
+			size = ldns_key_hmac_size(k);
 			memcpy(bin, ldns_key_hmac_key(k), size);
+			internal_data = 1;
 			break;
 	}
 	/* fourth the key bin material */
-	keybin = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_B64, size, bin);
-	LDNS_FREE(bin);
-	ldns_rr_push_rdf(pubkey, keybin);
+	if (internal_data) {
+		keybin = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_B64, size, bin);
+		LDNS_FREE(bin);
+		ldns_rr_push_rdf(pubkey, keybin);
+	}
 	return pubkey;
 }
 
@@ -1099,4 +1146,22 @@ ldns_read_anchor_file(const char *filename)
 	}
 }
 
+char *
+ldns_key_get_file_base_name(ldns_key *key)
+{
+	ldns_buffer *buffer;
+	char *file_base_name;
+	
+	buffer = ldns_buffer_new(255);
+	ldns_buffer_printf(buffer, "K");
+	ldns_rdf2buffer_str_dname(buffer, ldns_key_pubkey_owner(key));
+	ldns_buffer_printf(buffer,
+	                   "+%03u+%05u",
+			   ldns_key_algorithm(key),
+			   ldns_key_keytag(key));
+	file_base_name = strdup(ldns_buffer_export(buffer));
+	ldns_buffer_free(buffer);
+	return file_base_name;
+}
+
 #endif /* HAVE_SSL */
