@@ -60,10 +60,10 @@ ldns_tsig_prepare_pkt_wire(uint8_t *wire, size_t wire_len, size_t *result_len)
 	uint16_t ns_count;
 	uint16_t ar_count;
 	ldns_rr *rr;
-	
+
 	size_t pos;
 	uint16_t i;
-	
+
 	ldns_status status;
 
 	/* fake parse the wire */
@@ -71,7 +71,7 @@ ldns_tsig_prepare_pkt_wire(uint8_t *wire, size_t wire_len, size_t *result_len)
 	an_count = LDNS_ANCOUNT(wire);
 	ns_count = LDNS_NSCOUNT(wire);
 	ar_count = LDNS_ARCOUNT(wire);
-	
+
 	if (ar_count > 0) {
 		ar_count--;
 	} else {
@@ -79,7 +79,7 @@ ldns_tsig_prepare_pkt_wire(uint8_t *wire, size_t wire_len, size_t *result_len)
 	}
 
 	pos = LDNS_HEADER_SIZE;
-	
+
 	for (i = 0; i < qd_count; i++) {
 		status = ldns_wire2rr(&rr, wire, wire_len, &pos, LDNS_SECTION_QUESTION);
 		if (status != LDNS_STATUS_OK) {
@@ -87,7 +87,7 @@ ldns_tsig_prepare_pkt_wire(uint8_t *wire, size_t wire_len, size_t *result_len)
 		}
 		ldns_rr_free(rr);
 	}
-	
+
 	for (i = 0; i < an_count; i++) {
 		status = ldns_wire2rr(&rr, wire, wire_len, &pos, LDNS_SECTION_ANSWER);
 		if (status != LDNS_STATUS_OK) {
@@ -95,7 +95,7 @@ ldns_tsig_prepare_pkt_wire(uint8_t *wire, size_t wire_len, size_t *result_len)
 		}
 		ldns_rr_free(rr);
 	}
-	
+
 	for (i = 0; i < ns_count; i++) {
 		status = ldns_wire2rr(&rr, wire, wire_len, &pos, LDNS_SECTION_AUTHORITY);
 		if (status != LDNS_STATUS_OK) {
@@ -103,22 +103,22 @@ ldns_tsig_prepare_pkt_wire(uint8_t *wire, size_t wire_len, size_t *result_len)
 		}
 		ldns_rr_free(rr);
 	}
-	
+
 	for (i = 0; i < ar_count; i++) {
-		status = ldns_wire2rr(&rr, wire, wire_len, &pos, 
+		status = ldns_wire2rr(&rr, wire, wire_len, &pos,
 				LDNS_SECTION_ADDITIONAL);
 		if (status != LDNS_STATUS_OK) {
 			return NULL;
 		}
 		ldns_rr_free(rr);
 	}
-	
+
 	*result_len = pos;
 	wire2 = LDNS_XMALLOC(uint8_t, *result_len);
 	memcpy(wire2, wire, *result_len);
-	
+
 	ldns_write_uint16(wire2 + LDNS_ARCOUNT_OFF, ar_count);
-	
+
 	return wire2;
 }
 
@@ -126,10 +126,15 @@ ldns_tsig_prepare_pkt_wire(uint8_t *wire, size_t wire_len, size_t *result_len)
 static const EVP_MD *
 ldns_digest_function(char *name)
 {
-	/* TODO replace with openssl's EVP_get_digestbyname
-	        (need init somewhere for that)
-	*/
-	if (strlen(name) == 10 && strncasecmp(name, "hmac-sha1.", 9) == 0)
+	/* these are the mandatory algorithms from RFC4635 */
+	/* The optional algorithms are not yet implemented */	
+	if (strlen(name) == 12 && strncasecmp(name, "hmac-sha256.", 11) == 0) {
+#ifdef SHA256_DIGEST_LENGTH
+		return EVP_sha256();
+#else
+		return NULL;	
+#endif		
+	} else if (strlen(name) == 10 && strncasecmp(name, "hmac-sha1.", 9) == 0)
 		return EVP_sha1();
 	else if (strlen(name) == 25 && strncasecmp(name,
 		     "hmac-md5.sig-alg.reg.int.", 25) == 0)
@@ -182,8 +187,8 @@ ldns_tsig_mac_new(ldns_rdf **tsig_mac, uint8_t *pkt_wire, size_t pkt_wire_size,
 	
 	/* prepare the key */
 	key_bytes = LDNS_XMALLOC(unsigned char, 
-			b64_pton_calculate_size(strlen(key_data)));
-	key_size = b64_pton(key_data, key_bytes, strlen(key_data) * 2);
+			ldns_b64_pton_calculate_size(strlen(key_data)));
+	key_size = ldns_b64_pton(key_data, key_bytes, strlen(key_data) * 2);
 	if (key_size < 0) {
 		/* LDNS_STATUS_INVALID_B64 */
 		return LDNS_STATUS_INVALID_B64;
